package com.uas.search.jms;

import static com.uas.search.constant.SearchConstants.GOODS_INDEX_TASK;
import static com.uas.search.constant.SearchConstants.ORDERS_INDEX_TASK;
import static com.uas.search.constant.SearchConstants.PRODUCTS_INDEX_TASK;
import static com.uas.search.constant.SearchConstants.STANDARD_INDEX_TASK;

import com.uas.search.constant.SearchConstants;
import com.uas.search.constant.model.SPage;
import com.uas.search.schedule.Executable;
import com.uas.search.schedule.TaskInformation;
import com.uas.search.schedule.TaskService;
import com.uas.search.service.IndexService;
import com.uas.search.util.CollectionUtils;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**对数据库的消息队列进行实时监听
 *
 * @author sunyj
 * @since 2017/10/11 16:20
 */
@Service
public class JmsListener {

    /**
     * 第一次执行的延迟时间间隔为 3 秒，以防止日志交杂错乱，不好查看
     */
    private static final long INITIAL_DELAY = 3000;
    /**
     * 两次任务之间的等待时间间隔为 1 秒钟
     */
    private static final long INTERVAL = 1000;

    /**
     * 基础索引任务
     * STANDARD_INDEX_TASK 标准器件类索引任务，
     * GOODS_INDEX_TASK 在售商品类索引任务，
     * PRODUCTS_INDEX_TASK 物料索引任务，
     * ORDERS_INDEX_TASK 订单类索引任务
     */
    private static final String[] BASKTASKS= {STANDARD_INDEX_TASK, GOODS_INDEX_TASK,
        PRODUCTS_INDEX_TASK, ORDERS_INDEX_TASK};

    // 物料类包含的索引名称
    private static final String[] STANDARD_INDEX_TABLES = {SearchConstants.KIND_TABLE_NAME, SearchConstants.BRAND_TABLE_NAME, SearchConstants.COMPONENT_TABLE_NAME};

    // 在售商品类包含的索引名称
    private static final String[] GOODS_INDEX_TABLES = {SearchConstants.GOODS_TABLE_NAME, SearchConstants.PCB_GOODS_TABLE_NAME};

    // 物料类包含的索引名称
    private static final String[] PRODUCTS_INDEX_TABLES = {SearchConstants.PRODUCTS_PRIVATE_TABLE_NAME, SearchConstants.PRODUCTS_USERS_TABLE_NAME};

    // 物料类包含的索引名称
    private static final String[] ORDERS_INDEX_TABLES = {SearchConstants.ORDER_TABLE_NAME, SearchConstants.ORDER_INVOICE_TABLE_NAME,
                        SearchConstants.PURCHASE_TABLE_NAME, SearchConstants.PURCHASE_INVOICE_TABLE_NAME};

    @Autowired
    private LuceneMessageDao luceneMessageDao;

    @Autowired
    private IndexService indexService;

    @Autowired
    private TaskService taskService;

    @Autowired
    private LuceneMessageService luceneMessageService;

    @Autowired
    private QueueMessageParser queueMessageParser;

    private TaskInformation taskInformation;

    private Logger logger = LoggerFactory.getLogger(getClass());


    /**
     * 开启实时更新索引
     *
     * @param interval
     *            每次接收到jms消息后等待的时间（秒）
     *
     * @param taskName 启动任务名称
     * @return 开启成功与否的提示信息
     */
    public void start(Long interval, String taskName) {
        if (interval == null){
            interval = INTERVAL;
        } else {
            interval *= 1000;
        }
        if (interval <= 0) {
            throw new IllegalArgumentException("interval 不合法：" + interval);
        }

        if (isRunning(taskName)) {
            throw new IllegalStateException("已存在运行的索引" + taskInformation.getTitle() + "服务");
        }

        if (!taskService.isIdle()) {
            throw new IllegalStateException("任务池已满");
        }

        // 启动默认索引更新任务
        if (StringUtils.isEmpty(taskName)) {
            for (String baseTask : BASKTASKS) {
                if (isRunning(baseTask)) {
                    throw new IllegalStateException("已存在运行的" + baseTask + "索引实时更新服务");
                } else {
                    createTask(interval, baseTask);
                }
            }
        // 启动指定索引更新任务
        } else {

        }

//        try {
//            createTask(interval, "实时更新");
//        } catch (Throwable e) {
//            if (taskInformation != null) {
//                taskInformation = null;
//            }
//            throw new IllegalStateException("开启失败", e);
//        }
    }

    /**
     * 创建索引更新任务
     * @param interval 任务间隔时间
     * @param title 任务名称
     */
    private void createTask(Long interval, final String title) {
        Executable command = new Executable() {
            @Override
            public String execute() {
                long startTime = System.currentTimeMillis();
                String[] tablenames = null;
                switch (title) {
                    case STANDARD_INDEX_TASK:
                        tablenames = STANDARD_INDEX_TABLES;
                        break;
                    case GOODS_INDEX_TASK:
                        tablenames = GOODS_INDEX_TABLES;
                        break;
                    case PRODUCTS_INDEX_TASK:
                        tablenames = PRODUCTS_INDEX_TABLES;
                        break;
                    case ORDERS_INDEX_TASK:
                        tablenames = ORDERS_INDEX_TABLES;
                        break;
                    default: break;
                }
                SPage<LuceneMessage> sPage = luceneMessageService.findAll(1, 100, tablenames);
                List<LuceneMessage> luceneMessages = sPage.getContent();
                if (CollectionUtils.isEmpty(luceneMessages)) {
                    return "无消息";
                }
                logger.info(title + "获取luceneMessages" + luceneMessages.size() + "条,耗时：" + (System.currentTimeMillis() - startTime));
                // 按操作类型分组
                long indexTime = System.currentTimeMillis();
                Map<String, List<LuceneMessage>> luceneMessageMap = luceneMessages.stream().collect(Collectors.groupingBy(LuceneMessage :: getMethodType));
                List<LuceneMessage> insertMessage = luceneMessageMap.get("insert");
                List<LuceneMessage> updateMessage = luceneMessageMap.get("update");
                List<LuceneMessage> deleteMessage = luceneMessageMap.get("delete");

                List<Long> failedIds = new ArrayList<>();
                if (!CollectionUtils.isEmpty(insertMessage)) {
                    failedIds = process(insertMessage);
                    logger.info("insert索引" + insertMessage.size() + "条,失败" + failedIds.size() + "耗时" + (System.currentTimeMillis() - indexTime));
                }

                if (!CollectionUtils.isEmpty(updateMessage)) {
                    failedIds = process(updateMessage);
                    logger.info("update索引" + updateMessage.size() + "条,失败" + failedIds.size() + "耗时：" + (System.currentTimeMillis() - indexTime));
                }

                if (!CollectionUtils.isEmpty(deleteMessage)) {
                    failedIds = process(deleteMessage);
                    logger.info("delete索引" + deleteMessage.size() + "条,失败" + failedIds.size() + "耗时" + (System.currentTimeMillis() - indexTime));
                }

                if (!CollectionUtils.isEmpty(failedIds)) {
                    long retryTime = System.currentTimeMillis();
                    luceneMessageDao.updateReTryCount(failedIds);
                    logger.info("处理失败：" + failedIds.size() + "条，update retrycount耗时" + (System.currentTimeMillis() - retryTime));
                }

                logger.info("处理luceneMessages" + luceneMessages.size() + "条,耗时：" + (System.currentTimeMillis() - startTime));
                // 如果消息不止一页，立即消费下一页
                if (sPage.getTotalPage() > 1){
                    execute();
                }
                return "正常";
            }
        };
        taskInformation = new TaskInformation(title, command, INITIAL_DELAY, interval, TaskInformation.ScheduleType.FixedDelay);
        taskService.newTask(taskInformation);
        if (!taskService.isStopped(taskInformation)) {
            taskService.stop(taskInformation);
        }
        taskService.start(taskInformation);
    }
    /**
     * 关闭实时更新索引服务
     *
     * @return 关闭成功与否的提示信息
     */
    public void stop(String title) {
        String[] defaultTiltes = {STANDARD_INDEX_TASK, GOODS_INDEX_TASK, PRODUCTS_INDEX_TASK, ORDERS_INDEX_TASK};
        if (StringUtils.isEmpty(title)) {
            if (!taskService.isStopped()) {
                taskService.stop();
            }
        } else if (!isRunning(title)) {
            throw new IllegalStateException("索引实时更新服务未开启或已关闭");
        } else {
            taskService.remove(taskInformation.getCode());
            taskInformation = null;
            if (!taskService.isStopped(taskInformation)) {
                taskService.stop(taskInformation);
            }
            taskService.start(null);
        }
    }

    /**
     * @param title 索引名称
     * @return 索引实时更新服务是否正在运行
     */
    public boolean isRunning(String title) {
        return taskInformation != null && taskService.exist(StringUtils.isEmpty(title) ? taskInformation.getTitle() : title);
    }

    /**
     * 对得到的队列消息进行解析，之后根据解析出来的对象，对lucene索引进行添加、更新或删除操作
     *
     * @param luceneMessages 消息
     */
    private List<Long> process(List<LuceneMessage> luceneMessages) {
        List<Long> successIds = new ArrayList<>();
        List<Long> failedIds = new ArrayList<>();
        for (LuceneMessage luceneMessage : luceneMessages) {
            try {
                ParsedQueueMessage parsedQueueMessage = queueMessageParser.parse(luceneMessage.getTableName(), luceneMessage.getDataId(), luceneMessage.getMethodType(), luceneMessage.getData());
                List<Object> maintainedObjects = indexService.maintainIndexes(parsedQueueMessage);
                logger.info("Maintained... " + maintainedObjects);
                successIds.add(luceneMessage.getId());
            } catch (Exception e) {
                failedIds.add(luceneMessage.getId());
            }
        }
        if (!CollectionUtils.isEmpty(successIds)) {
            long successTime = System.currentTimeMillis();
            luceneMessageDao.dequeueLuceneMessages(successIds);
            luceneMessageDao.deleteByIds(successIds);
            logger.info("处理成功" + successIds.size() + "条，耗时" + (System.currentTimeMillis() - successTime));
        }
        return failedIds;
    }
}
