package com.uas.search.jms;

import com.uas.search.annotation.NotEmpty;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.jpa.repository.query.Procedure;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author sunyj
 * @since 2016年7月7日 下午5:52:52
 */
@Repository
public interface LuceneMessageDao
        extends JpaSpecificationExecutor<LuceneMessage>, JpaRepository<LuceneMessage, Long> {

    /**
     * 获取指定数目的数据
     *
     * @param start 开始的记录
     * @param size 指定数目
     * @return 数据
     */
    @Query(value = "select * from lucene$message where me_retry_count < 5 order by me_priority desc, me_id limit ?1, ?2", nativeQuery = true)
    List<LuceneMessage> findList(@NotEmpty("start") Integer start, @NotEmpty("size") Integer size);

    /**
     * 获取指定表名、数目的数据
     *
     * @param start 开始的记录
     * @param size 指定数目
     * @param tableNames 表名
     * @return 数据
     */
    @Query(value = "select * from lucene$message where me_retry_count < 5 and me_table_name in :tableNames order by me_priority desc, me_id limit :start, :size", nativeQuery = true)
    List<LuceneMessage> findListByTableNames(@Param("start") Integer start, @Param("size") Integer size, @Param("tableNames") String[] tableNames);

    /**
     * 统计行数
     * @return 行数
     */
    @Query(value = "select count(1) from lucene$message where me_retry_count < 5", nativeQuery = true)
    long count();

    /**
     * 统计指定表名的行数
     * @param tableNames
     * @return
     */
    @Query(value = "select count(1) from lucene$message where me_retry_count < 5 and me_table_name in :tableNames", nativeQuery = true)
    long countByTableNames(@Param("tableNames") String[] tableNames);

    /**
     * 出队消息
     * @param id 消息 id
     */
    @Transactional
    @Procedure(procedureName = "dequeue_lucene_message")
    void dequeueLuceneMessage(@NotEmpty("id") Long id);

    /**
     * 批量将已处理消息插入消息历史
     * @param ids
     */
    @Modifying
    @Transactional
    @Query(value = "insert into lucene$message_history (mh_dequeue_time, me_id, me_table_name, me_method_type, me_data_id, me_data, me_priority, me_create_time) select now(), me_id, me_table_name, me_method_type, me_data_id, me_data, me_priority, me_create_time from lucene$message where me_id in :ids", nativeQuery = true)
    void dequeueLuceneMessages(@Param("ids") List<Long> ids);

    /**
     * 批量删除已处理消息
     * @param ids
     */
    @Modifying
    @Transactional
    @Query(value = "delete from lucene$message where me_id in :ids", nativeQuery = true)
    void deleteByIds(@Param("ids") List<Long> ids);

    /**
     * 批量更新处理失败消息重试次数+1
     * @param ids
     */
    @Modifying
    @Transactional
    @Query(value = "update lucene$message set me_retry_count = me_retry_count+1 where me_id in :ids", nativeQuery = true)
    void updateReTryCount(@Param("ids") List<Long> ids);
}
