package com.uas.search.service;

import com.uas.search.annotation.NotEmpty;
import com.uas.search.constant.model.CollectField;
import com.uas.search.constant.model.PageParams;
import com.uas.search.constant.model.PageParams.FilterField;
import com.uas.search.constant.model.SPage;
import com.uas.search.exception.SearchException;
import com.uas.search.model.Brand;
import com.uas.search.model.Component;
import com.uas.search.model.Goods;
import com.uas.search.model.Kind;
import com.uas.search.model.PCBGoods;
import com.uas.search.model.V_Products;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 搜索服务的接口
 * 
 * @author suntg
 * @since 2016年7月29日下午4:58:45
 */
public interface SearchService {

	/**
	 * 根据关键字搜索产品类目id
	 * 
	 * @param keyword
	 *            关键词
	 * @param page
	 *            页码
	 * @param size
	 *            页大小
	 * @return 符合条件的类目id
	 */
	public SPage<Long> getKindIds(String keyword, Integer page, Integer size) throws IOException;

	/**
	 * 根据关键词搜索产品类目
	 * 
	 * @param keyword
	 *            关键词
	 * @param page
	 *            页码
	 * @param size
	 *            页大小
	 * @return
	 */
	public SPage<Map<String, Object>> getKinds(String keyword, Integer page, Integer size) throws IOException;

	/**
	 * 根据关键词搜索产品品牌id
	 * 
	 * @param keyword
	 *            关键词
	 * @param page
	 *            页码
	 * @param size
	 *            页大小
	 * @return 符合条件的品牌id
	 */
	public SPage<Long> getBrandIds(String keyword, Integer page, Integer size) throws IOException;

	/**
	 * 根据关键词搜索产品品牌
	 * 
	 * @param keyword
	 *            关键词
	 * @param page
	 *            页码
	 * @param size
	 *            页大小
	 * @return 符合条件的品牌
	 */
	public SPage<Map<String, Object>> getBrands(String keyword, Integer page, Integer size) throws IOException;

	/**
	 * 根据关键词搜索品牌（精确命中后，提供卖家信息）
	 * @param keyword 关键词
	 * @param page 页码
	 * @param size 页数
	 * @return
	 * @throws IOException
	 */
	SPage<Map<String, Object>> getBrandsAndSellers(String keyword, Integer page, Integer size, String storeType) throws IOException;

	/**
	 * 根据关键词搜索品牌（精确命中后，提供卖家信息）
	 * @param keyword 关键词
	 * @param page 页码
	 * @param size 页数
	 * @return
	 * @throws IOException
	 */
	Map<String, Object> getSellersWithKind(String keyword, Integer page, Integer size, String storeType) throws IOException;

	/**
	 * 根据关键词搜索产品（关键词可能是器件、类目、品牌，甚至可能是类目、品牌的混合）
	 * 
	 * @param keyword
	 *            关键词
	 * @param params
	 *            翻页、过滤等信息
	 *            <p>
	 *            关于过滤，通过键值对指定过滤条件，键为
	 *            {@link com.uas.search.constant.model.PageParams.FilterField}
	 *            ，值的类型由键决定：
	 *            </p>
	 * 
	 *            <table border=1 cellpadding=5 cellspacing=0 summary=
	 *            "Fields and types">
	 *            <tr>
	 *            <th>Field</th>
	 *            <th>Type</th>
	 *            </tr>
	 *            <tr>
	 *            <td>COMPONENT_KINDID</td>
	 *            <td>Long</td>
	 *            </tr>
	 *            <tr>
	 *            <td>COMPONENT_BRANDID</td>
	 *            <td>Long</td>
	 *            </tr>
	 *            <tr>
	 *            <td>COMPONENT_PROPERTIES</td>
	 *            <td>键值对，键值分别为属性id、属性值</td>
	 *            </tr>
	 *            <tr>
	 *            <td>COMPONENT_HAS_RESERVE</td>
	 *            <td>Boolean</td>
	 *            </tr>
	 *            <tr>
	 *            <td>COMPONENT_HAS_SAMPLE</td>
	 *            <td>Boolean</td>
	 *            </tr>
	 *            <tr>
	 *            <td>COMPONENT_HAS_ORIGINAL</td>
	 *            <td>Boolean</td>
	 *            </tr>
	 *            <tr>
	 *            <td>COMPONENT_HAS_INACTION_STOCK</td>
	 *            <td>Boolean</td>
	 *            </tr>
	 *            </table>
	 * 
	 * @return
	 */
	public Map<String, Object> getComponentIds(String keyword, PageParams params) ;

	/**
	 * 根据产品搜索获取产品类目id的统计
	 * 
	 * @param keyword
	 * @param brandId
	 *            （可选）
	 * @return
	 */
	public Set<Long> getKindIdsBySearchComponent(String keyword, String brandId) ;

	/**
	 * 根据产品搜索获取产品类目的统计
	 * 
	 * @param keyword
	 * @param brandId
	 *            （可选）
	 * @return
	 */
	public Set<Map<String, Object>> getKindsBySearchComponent(String keyword, String brandId) ;

	/**
	 * 根据产品搜索获取产品品牌id的统计
	 * 
	 * @param keyword
	 * @param kindId
	 *            （可选）
	 * @return
	 */
	public Set<Long> getBrandIdsBySearchComponent(String keyword, String kindId) ;

	/**
	 * 根据产品搜索获取产品品牌的统计
	 * 
	 * @param keyword
	 * @param kindId
	 *            （可选）
	 * @return
	 */
	public Set<Map<String, Object>> getBrandsBySearchComponent(String keyword, String kindId) ;

	/**
	 * 根据输入获取联想词（包括器件、类目、品牌，按顺序获取，数量不足，才会获取下一个）
	 * 
	 * @param keyword
	 * @param size 指定的联想词数目
	 * @return
	 */
	public List<String> getSimilarKeywords(String keyword, Integer size) ;

	/**
	 * 根据输入的原厂型号获取联想词
	 * 
	 * @param componentCode
	 * @param size 指定的联想词数目
	 * @return 包括id、uuid、code
	 */
	public List<Map<String, Object>> getSimilarComponents(String componentCode, Integer size) ;

	/**
	 * 根据输入的物料型号获取联想词
	 *
	 * @param code
	 * @param size 指定的联想词数目
	 * @return 包括id、uuid、code
	 */
	List<Map<String, Object>> getSimilarProducts(String code, Integer size) ;

	/**
	 * 根据输入的品牌获取联想词
	 * 
	 * @param brandName
	 * @param size 指定的联想词数目
	 * @return 包括id、uuid、nameCn、nameEn
	 */
	public List<Map<String, Object>> getSimilarBrands(String brandName, Integer size) throws IOException;

	/**
	 * 根据输入的类目名获取联想词
	 * 
	 * @param kindName
	 * @param size 指定的联想词数目
	 * @return 包括id、nameCn、level、isLeaf
	 */
	public List<Map<String, Object>> getSimilarKinds(String kindName, Integer size) throws IOException;

	/**
	 * 根据输入的类目名获取末级类目联想词
	 * 
	 * @param kindName
	 * @param size 指定的联想词数目
	 * @return 包括id、nameCn、level、isLeaf
	 */
	public List<Map<String, Object>> getSimilarLeafKinds(String kindName, Integer size) throws IOException;

	/**
	 * 根据输入的类目名和指定的类目级别获取联想词
	 * 
	 * @param kindName
	 * @param level
	 * @param size 指定的联想词数目
	 * @return 包括id、nameCn、level、isLeaf
	 */
	public List<Map<String, Object>> getSimilarKindsByLevel(String kindName, Short level, Integer size) throws IOException;

	/**
	 * 根据类目id、属性id、属性值获取联想词
	 * 
	 * @param kindId
	 *            类目id
	 * @param propertyId
	 *            属性id
	 * @param keyword
	 *            （可选） 属性值（部分字符）
	 * @param topNum
	 *            （可选） 获取的最大数目
	 * @return 相似的属性值，包括propertyValue
	 */
	public List<Map<String, String>> getSimilarPropertyValues(Long kindId, Long propertyId, String keyword, Long topNum);

	/**
	 * 获取在售商品型号联想词
	 * @param keyword 关键词
	 * @param storeType 店铺类型
	 * @param size 个数
	 * @throws IOException
	 */
	public Set<Map<String, Object>> getSimilarGoodsCode(String keyword, String storeType, Integer size) throws IOException;

	/**
	 * 获取在售商品品牌联想词
	 * @param keyword 关键词
	 * @param storeType 店铺类型
	 * @param size 个数
	 * @throws IOException
	 */
	public Set<Map<String, Object>> getSimilarGoodsBrand(String keyword, String storeType, Integer size) throws IOException;

	/**
	 * 获取在售商品类目联想词
	 * @param keyword 关键词
	 * @param storeType 店铺类型
	 * @param size 个数
	 * @throws IOException
	 */
	public Set<Map<String, Object>> getSimilarGoodsKind(String keyword, String storeType, Integer size) throws IOException;

	/**
	 * 根据关键词搜索批次（关键词可能是器件、类目、品牌）
	 * 
	 * @param keyword
	 *            关键词
	 * @param pageParams
	 *            翻页、过滤、排序等信息
	 *            <p>
	 *            关于过滤，通过键值对指定过滤条件，键为
	 *            {@link com.uas.search.constant.model.PageParams.FilterField}
	 *            ，值的类型由键决定：
	 *            </p>
	 *
	 *            <table border=1 cellpadding=5 cellspacing=0 summary=
	 *            "Fields and types">
	 *            <tr>
	 *            <th>Field</th>
	 *            <th>Type</th>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_KINDID</td>
	 *            <td>List(Long)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_BRANDID</td>
	 *            <td>List(Long)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_STORE_TYPE</td>
	 *            <td>List(String)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_CRNAME</td>
	 *            <td>List(String)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_MINPRICERMB</td>
	 *            <td>Double</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_MAXPRICERMB</td>
	 *            <td>Double</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_MINPRICEUSD</td>
	 *            <td>Double</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_MAXPRICEUSD</td>
	 *            <td>Double</td>
	 *            </tr>
	 *            </table>
	 *
	 * @return 器件id、批次id和分页信息
	 */
	public Map<String, Object> getGoodsIds(String keyword, PageParams pageParams, String storeType) throws IOException;

	/**
	 * 根据关键词搜索批次（关键词可能是器件、类目、品牌）
	 *
	 * @param keyword
	 *            关键词
	 * @param pageParams
	 *            翻页、过滤、排序等信息
	 *            <p>
	 *            关于过滤，通过键值对指定过滤条件，键为
	 *            {@link com.uas.search.constant.model.PageParams.FilterField}
	 *            ，值的类型由键决定：
	 *            </p>
	 *
	 *            <table border=1 cellpadding=5 cellspacing=0 summary=
	 *            "Fields and types">
	 *            <tr>
	 *            <th>Field</th>
	 *            <th>Type</th>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_KINDID</td>
	 *            <td>List(Long)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_BRANDID</td>
	 *            <td>List(Long)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_STORE_TYPE</td>
	 *            <td>List(String)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_CRNAME</td>
	 *            <td>List(String)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_MINPRICERMB</td>
	 *            <td>Double</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_MAXPRICERMB</td>
	 *            <td>Double</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_MINPRICEUSD</td>
	 *            <td>Double</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_MAXPRICEUSD</td>
	 *            <td>Double</td>
	 *            </tr>
	 *            </table>
	 *
	 * @return 器件id、批次id和分页信息
	 */
	public Map<String, Object> getOldGoodsIds(String keyword, PageParams pageParams) throws IOException;

    /**
     * 根据关键词搜索 PCB 批次
     *
     * @param keyword
     *            关键词
     * @param pageParams
     *            翻页、过滤、排序等信息
     *            <p>
     *            关于过滤，通过键值对指定过滤条件，键为
     *            {@link com.uas.search.constant.model.PageParams.FilterField}
     *            ，值的类型由键决定：
     *            </p>
     *
     *            <table border=1 cellpadding=5 cellspacing=0 summary=
     *            "Fields and types">
     *            <tr>
     *            <th>Field</th>
     *            <th>Type</th>
     *            </tr>
     *            <tr>
     *            <td>GOODS_KINDID</td>
     *            <td>List(Long)</td>
     *            </tr>
     *            <tr>
     *            <td>GOODS_BRANDID</td>
     *            <td>List(Long)</td>
     *            </tr>
     *            <tr>
     *            <td>GOODS_CRNAME</td>
     *            <td>List(String)</td>
     *            </tr>
     *            <tr>
     *            <td>GOODS_MINPRICERMB</td>
     *            <td>Double</td>
     *            </tr>
     *            <tr>
     *            <td>GOODS_MAXPRICERMB</td>
     *            <td>Double</td>
     *            </tr>
     *            <tr>
     *            <td>GOODS_MINPRICEUSD</td>
     *            <td>Double</td>
     *            </tr>
     *            <tr>
     *            <td>GOODS_MAXPRICEUSD</td>
     *            <td>Double</td>
     *            </tr>
     *            </table>
     *
     * @return PCB id、批次 id 和分页信息
     */
    Map<String, Object> getPCBGoodsIds(String keyword, PageParams pageParams) throws IOException;

	/**
	 * 搜索批次时，统计指定信息
	 * 
	 * @param keyword
	 *            关键词
	 * @param collectedField
	 *            需要统计的信息
	 * @param filters
	 *            过滤条件，值的类型由键决定：
	 * 
	 *            <table border=1 cellpadding=5 cellspacing=0 summary=
	 *            "Fields and types">
	 *            <tr>
	 *            <th>Field</th>
	 *            <th>Type</th>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_KINDID</td>
	 *            <td>List(Long)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_BRANDID</td>
	 *            <td>List(Long)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_STORE_TYPE</td>
	 *            <td>List(String)</td>
	 *            </tr>
	 *            <tr>
	 *            <td>GOODS_CRNAME</td>
	 *            <td>List(String)</td>
	 *            </tr>
	 *            </table>
	 * 
	 * @return 统计的信息（由collectedField决定）
	 * 
	 *         <table border=1 cellpadding=5 cellspacing=0 summary=
	 *         "Collected fields and messages">
	 *         <tr>
	 *         <th>Collected field</th>
	 *         <th>Message</th>
	 *         </tr>
	 *         <tr>
	 *         <td>GOODS_KIND</td>
	 *         <td>ki_id、ki_name_cn</td>
	 *         </tr>
	 *         <tr>
	 *         <td>GOODS_BRAND</td>
	 *         <td>br_id、br_uuid、br_name_cn、br_name_en</td>
	 *         </tr>
	 *         <tr>
	 *         <td>GOODS_STORE_TYPE</td>
	 *         <td>store_type</td>
	 *         </tr>
	 *         <tr>
	 *         <td>GOODS_CRNAME</td>
	 *         <td>cr_name</td>
	 *         </tr>
	 *         </table>
	 */
	public List<Map<String, Object>> collectBySearchGoods(String keyword, CollectField collectedField,
			Map<FilterField, Object> filters) throws IOException;

    /**
     * 搜索 PCB 批次时，统计指定信息
     *
     * @param keyword
     *            关键词
     * @param collectedField
     *            需要统计的信息
     * @param filters
     *            过滤条件，值的类型由键决定：
     *
     *            <table border=1 cellpadding=5 cellspacing=0 summary=
     *            "Fields and types">
     *            <tr>
     *            <th>Field</th>
     *            <th>Type</th>
     *            </tr>
     *            <tr>
     *            <td>GOODS_KINDID</td>
     *            <td>List(Long)</td>
     *            </tr>
     *            <tr>
     *            <td>GOODS_BRANDID</td>
     *            <td>List(Long)</td>
     *            </tr>
     *            <tr>
     *            <td>GOODS_CRNAME</td>
     *            <td>List(String)</td>
     *            </tr>
     *            </table>
     *
     * @return 统计的信息（由collectedField决定）
     *
     *         <table border=1 cellpadding=5 cellspacing=0 summary=
     *         "Collected fields and messages">
     *         <tr>
     *         <th>Collected field</th>
     *         <th>Message</th>
     *         </tr>
     *         <tr>
     *         <td>GOODS_KIND</td>
     *         <td>ki_id、ki_name_cn</td>
     *         </tr>
     *         <tr>
     *         <td>GOODS_BRAND</td>
     *         <td>br_id、br_uuid、br_name_cn、br_name_en</td>
     *         </tr>
     *         <tr>
     *         <td>GOODS_CRNAME</td>
     *         <td>cr_name</td>
     *         </tr>
     *         </table>
     */
    List<Map<String, Object>> collectBySearchPCBGoods(String keyword, CollectField collectedField,
                                                   Map<FilterField, Object> filters) throws IOException;

	/**
	 * 根据id获取类目
	 * 
	 * @param id
	 * @return
	 */
	public Kind getKind(Long id) throws IOException;

	/**
	 * 根据id获取品牌
	 * 
	 * @param id
	 * @return
	 */
	public Brand getBrand(Long id) throws IOException;

	/**
	 * 根据id获取器件
	 * 
	 * @param id
	 * @return
	 */
	public Component getComponent(Long id) throws IOException;

	/**
	 * 根据id获取批次
	 * 
	 * @param id
	 * @return
	 */
	public Goods getGoods(String id) throws IOException;

    /**
     * 根据id获取 PCB 批次
     *
     * @param id
     * @return
     */
    public PCBGoods getPCBGoods(String id) throws IOException;

	/**
	 * 分页获取本地指定表的索引中的数据
	 * 
	 * @param tableName
	 *            表名
	 * @param keyword
	 *            关键词，可为空，默认正则搜索空串
	 * @param field
	 *            搜索字段，可为空，默认取各表索引中的id字段
	 * @param tokenized
	 *            是否分词，可为空，默认不分词
	 * @param page
	 *            页码
	 * @param size
	 *            分页大小
	 * @return
	 */
	public SPage<Object> getObjects(String tableName, String keyword, String field, Boolean tokenized, @NotEmpty("page") Integer page, @NotEmpty("size") Integer size) throws IOException;

	/**
	 * 查询物料
	 * @param enUU 企业UU
	 * @param keyword 关键词
	 * @param page  页码
	 * @param size  尺寸
	 * @param type  all 全部  standard 标准   nStandard 非标
	 * @param duplicate pcmpcode是否去重
	 * @return idPage
	 * @throws IOException 输入异常
	 */
	SPage<Long> getProductIds(Long enUU, String keyword, Integer page, Integer size, String type, Boolean duplicate) throws IOException;

	/**
	 * 查询物料(B2B)
	 * @param keyword 关键词
	 * @param page  页码
	 * @param size  尺寸
	 * @return idPage
	 * @throws IOException 输入异常
	 */
	SPage<Long> getProductIdsB2B(String keyword, String tableName, Integer page, Integer size,
		String keywordSearchColumns, String filters, String multiValueField, String sort, String notEqualFilters,
		String multiValueNotField) throws IOException;

	/**
	 * 根据关键词、索引名称、状态码搜索单据id
	 *
	 * @param keyword
	 *            可为空，模糊搜索关键词
	 * @param tableName
	 *            不为空，单据类型
	 * @param pageParams
	 *            可为空，翻页、过滤、排序、排除过滤等信息
	 *            <p>
	 *            关于过滤，通过键值对指定过滤条件，值的类型由键决定：
	 *            </p>
	 *
	 *            <table border=1 cellpadding=5 cellspacing=0 summary=
	 *            "Fields and types">
	 *            <tr>
	 *            <th>Description</th>
	 *            <th>Field</th>
	 *            <th>Type</th>
	 *            </tr>
	 *            <tr>
	 *            <td>状态、所属企业uu、其他状态（如已采纳、未采纳等）</td>
	 *            <td>键为数据库表中相应的字段名称</td>
	 *            <td>值为字段对应的值，若值有多个，则使用
	 *            {@link com.uas.search.constant.model.MultiValue}</td>
	 *            </tr>
	 *            <tr>
	 *            <td>开始时间、截止时间， 这两个参数用于对时间范围进行筛选</td>
	 *            <td>{@link com.uas.search.constant.SearchConstants}</td>
	 *            <td>Long</td>
	 *            </tr>
	 *            <tr>
	 *            <td>排序方式</td>
	 *            <td>{@link com.uas.search.constant.SearchConstants}</td>
	 *            <td>List({@link com.uas.search.constant.model.Sort})</td>
	 *            </tr>
	 *            </table>
	 *
	 *            <p>
	 *            关于排除过滤，键为数据库表中相应的字段名称，值为需排除的值，若值有多个，则使用
	 *            {@link com.uas.search.constant.model.MultiValue}
	 *            </p>
	 * @return 单据id
	 * @throws SearchException
	 */
	SPage<Long> searchIds(String keyword, Table_name tableName, PageParams pageParams) throws SearchException;

	/**
	 * 根据id获取物料
	 *
	 * @param id
	 * @return
	 */
    V_Products getProduct(Long id) throws IOException;

	/**
	 * 获取标准型号联想词
	 * @param keyword 关键词
	 * @param size 尺寸
	 * @return
	 */
	List<Map<String,Object>> getSimilarPCmpCodes(String keyword, Integer size);

	/**
	 * 获取类目联想词
	 * @param keyword 关键词
	 * @param size 尺寸
	 * @return
	 */
	List<Map<String,Object>> getSimilarKind(String keyword, Integer size);

	public enum Table_name {
		/**
		 * 物料
		 */
		V$PRODUCT$PRIVATE("v$product$private"),

		/**
		 * 个人物料
		 */
		PRODUCT$USERS("product$users");

		private Table_name(String phrase) {
			this.phrase = phrase;
		}

		private String phrase;

		public String value() {
			return phrase;
		}

		@Override
		public String toString() {
			return phrase;
		}
	}
}
