package com.uas.search.support;

import static com.uas.search.service.impl.IndexServiceImpl.PAGE_SIZE;
import static com.uas.search.service.impl.IndexServiceImpl.SINGLE_FILE_MAX_SIZE;

import com.alibaba.fastjson.JSONObject;
import com.uas.search.constant.model.PageParams;
import com.uas.search.service.JdbcService;
import com.uas.search.util.CollectionUtils;
import com.uas.search.util.SearchUtils;
import java.io.File;
import java.io.PrintWriter;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Sort;

/**
 * 下载线程
 *
 * @author sunyj
 * @since 2017/11/25 18:24
 * @version 2018/7/6 修改下载数据的策略为按id范围端下载
 */
public class DownloadService<T> {

    /**
     * 一次下载计数器，统计一次下载的总个数，包括各个线程的
     */
    private static long sizeCounter = 0;

    /**
     * 数据文件文件名格式
     */
    private static final String FILE_NAME_FORMAT = "%010d.txt";

    /**
     * @param id             线程 id
     * @param step           新增文件时，文件 id 的自增步长，（即线程数量）
     * @param startFileIndex 开始的文件
     * @param endFileIndex   结束的文件
     * @param tableName      要下载的表
     * @param trueTableName  真正的表（可以不为实体表）
     * @param sortField      排序字段
     * @param jdbcService    jdbcService
     * @return 下载的数量
     */
    public long download(int id, int step, int startFileIndex, int endFileIndex, String tableName, String trueTableName, String sortField, JdbcService jdbcService) {
        // 本次线程实际下载数据总数
        long size = 0L;
        Logger logger = LoggerFactory.getLogger(getClass());
        String name = "Thread_" + id + "_" + startFileIndex;
        File file = null;
        PrintWriter printWriter = null;
        try {
            if (SINGLE_FILE_MAX_SIZE % PAGE_SIZE != 0) {
                throw new IllegalArgumentException("SINGLE_FILE_MAX_SIZE 并非 PAGE_SIZE 的整数倍：" + SINGLE_FILE_MAX_SIZE + "/" + PAGE_SIZE);
            }
            if (startFileIndex > endFileIndex) {
                logger.info(name + " startFileIndex " + startFileIndex + " 超过 endFileIndex " + endFileIndex);
                return size;
            }
            Long startTime = System.currentTimeMillis();
            logger.info(name + " 下载 " + tableName + "...");

            // 分页获取数据
            PageParams pageParams = new PageParams();
            pageParams.setPage((startFileIndex - 1) * SINGLE_FILE_MAX_SIZE / PAGE_SIZE + 1);
            pageParams.setSize(PAGE_SIZE);

            Long maxId = jdbcService.getMaxId(trueTableName, sortField);
            // 理论上的总文件数
            int totalFiles = (int) (maxId / SINGLE_FILE_MAX_SIZE + 1);
            if (startFileIndex > totalFiles) {
                logger.info(name + " startFileIndex 超过 " + totalFiles);
                return size;
            }
            // 实际的最后一个文件
            endFileIndex = endFileIndex < totalFiles ? endFileIndex : totalFiles;

            File dataDir = new File(SearchUtils.getDataPath(tableName));
            if (!dataDir.exists()) {
                if (!dataDir.mkdirs()) {
                    logger.error("创建文件夹失败：" + dataDir.getPath());
                }
            }
            int fileIndex = startFileIndex;
            String fileName = String.format(FILE_NAME_FORMAT, fileIndex);
            // 当前文件已获取id区间数
            int count = 0;
            // 当前文件已获取实际数据条数
            int fileDataSize = 0;
            while (endFileIndex >= fileIndex) {
                Long startId = (long) ((pageParams.getPage() - 1) * pageParams.getSize() + 1);
                Long endId = (long) (pageParams.getPage() * pageParams.getSize());
                // 获取数据
                List<T> content = jdbcService.getData(trueTableName, startId, endId, sortField, Sort.Direction.ASC.name());
                // 数据以 JSON 格式写入文件
                if (!CollectionUtils.isEmpty(content)) {
                    // 有数据，并且还没有创建新文件时，进行创建新文件
                    if (fileDataSize == 0) {
                        close(printWriter);
                        fileName = String.format(FILE_NAME_FORMAT, fileIndex);
                        file = new File(dataDir, fileName);
                        printWriter = new PrintWriter(file);
                    }

                    for (T element : content) {
                        println(printWriter, element);
                    }

                    size += content.size();
                    fileDataSize += content.size();
                }
                count += PAGE_SIZE;
                logger.info("{} {} download id:{}~{}, current data size: {}, total data size: {}", name, fileName, startId, endId, content.size(), size);
                pageParams.setPage(pageParams.getPage() + 1);

                // 一个文件的获取id数达到指定值，写入新文件
                if (count == SINGLE_FILE_MAX_SIZE) {
                    count = 0;
                    fileDataSize = 0;
                    fileIndex += step;
                    pageParams.setPage(pageParams.getPage() + (step -1) * SINGLE_FILE_MAX_SIZE / PAGE_SIZE);
                }
            }
            logger.info(String.format("%s 下载完成，耗时%.2fs\n ", name, (System.currentTimeMillis() - startTime) / 1000.0));
        } catch (Throwable e) {
            close(printWriter);
            // 出现错误时，删除最近出错的文件
            if (file != null) {
                if (!file.delete()) {
                    logger.error("出错时，清理出错文件失败：" + file.getPath());
                }
            }
            logger.error(name + " " + tableName + " 下载失败", e);
        } finally {
            close(printWriter);
        }
        return size;
    }

    protected void println(PrintWriter printWriter, T element) {
        printWriter.println(JSONObject.toJSONString(element));
    }

    private void close(PrintWriter printWriter) {
        if (printWriter != null) {
            printWriter.flush();
            printWriter.close();
        }
    }

}
