package com.uas.search.console.b2b.jms;

import com.uas.search.b2b.model.SPage;
import com.uas.search.console.b2b.schedule.model.TaskInformation;
import com.uas.search.console.b2b.schedule.service.Executable;
import com.uas.search.console.b2b.schedule.service.TaskService;
import com.uas.search.console.b2b.service.IndexService;
import com.uas.search.console.b2b.util.ContextUtils;
import com.uas.search.console.b2b.util.SearchConstants.DataSourceQualifier;
import com.uas.search.util.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 对数据库的消息队列进行实时监听
 * 
 * @author sunyj
 * @since 2016年11月9日 上午11:29:33
 */
@Service
public class JmsListener {

    /**
     * 第一次执行的延迟时间间隔为 3 秒，以防止日志交杂错乱，不好查看
     */
    private static final long INITIAL_DELAY = 3000;
    /**
     * 两次任务之间的等待时间间隔为 1 秒钟
     */
    private static final long INTERVAL = 1000;

	@Autowired
	private IndexService indexService;

    @Autowired
    private TaskService taskService;

    @Autowired
    private LuceneMessageService luceneMessageService;

	@Autowired
	private QueueMessageParser queueMessageParser;

	/**
	 * 消息队列的消费者
	 */
	private Map<DataSourceQualifier, TaskInformation> consumers = new ConcurrentHashMap<>();

	private Logger logger = LoggerFactory.getLogger(getClass());

	/**
	 * 开启实时更新索引
	 * 
	 * @param waitInterval
	 *            每次接收到jms消息后等待的时间（秒）
	 * @return 开启成功与否的提示信息
	 */
	public String start(HashSet<DataSourceQualifier> dataSourceQualifiers, Long waitInterval) {
		// 不指定数据源标识时，开启所有监听器
		if (CollectionUtils.isEmpty(dataSourceQualifiers)) {
			dataSourceQualifiers = allDataSourceTypes();
		}
		String message = "";
		try {
			for (DataSourceQualifier qualifier : dataSourceQualifiers) {
				message += "\n" + startListener(qualifier, waitInterval);
			}
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
		}
		return message;
	}

	/**
	 * 添加监听器
	 * 
	 * @param dataSourceQualifier 数据源标识
	 * @param waitInterval
	 */
	private String startListener(DataSourceQualifier dataSourceQualifier, Long waitInterval) {
        if(waitInterval == null){
            waitInterval = INTERVAL;
        }else{
            waitInterval *= 1000;
        }
        if(waitInterval <= 0){
            throw new IllegalArgumentException("waitInterval 不合法：" + waitInterval);
        }
		String message;
		if (isRunning(dataSourceQualifier)) {
			message = "已存在运行的索引实时更新服务：" + dataSourceQualifier.getQualifier();
			logger.warn(message);
			return message;
		}

		try {
			TaskInformation consumer = getMessageConsumer(dataSourceQualifier, waitInterval);
			consumers.put(dataSourceQualifier, consumer);
            taskService.newTask(consumer);
            if (!taskService.isStopped()) {
                taskService.stop();
            }
            taskService.start();
			message = "索引实时更新服务开启成功：" + dataSourceQualifier.getQualifier();
			logger.info(message);
		} catch (Exception e) {
			message = "索引实时更新服务开启失败：" + dataSourceQualifier.getQualifier();
			logger.error(message, e);
			try {
				closeConsumer(dataSourceQualifier);
			} catch (Exception e1) {
				logger.error("", e1);
			}
		}
		return message;
	}

	/**
     * @param dataSourceQualifier 数据源标识
     * @param waitInterval
	 */
	private TaskInformation getMessageConsumer(final DataSourceQualifier dataSourceQualifier, Long waitInterval) {
        String title = "实时更新[" + dataSourceQualifier + "]";
        Executable command = new Executable() {
            @Override
            public String execute() {
                SPage<LuceneMessageInterface> sPage = luceneMessageService.findAll(dataSourceQualifier, 1, 100);
                List<LuceneMessageInterface> luceneMessages = sPage.getContent();
                if (CollectionUtils.isEmpty(luceneMessages)) {
                    return "无消息";
                }
                for(LuceneMessageInterface luceneMessage : luceneMessages){
					logger.info("LuceneMessage.id..." + luceneMessage.getId());
                    try {
                        process(luceneMessage);
                    } catch (Throwable e) {
                        logger.error("", e);
                    }
                }
                // 如果消息不止一页，立即消费下一页
                if(sPage.getTotalPage() > 1){
                    execute();
                }
                return "正常";
            }
        };
        return new TaskInformation(title, command, INITIAL_DELAY, waitInterval, TaskInformation.ScheduleType.FixedDelay);
	}

	/**
	 * 关闭实时更新索引服务
	 * 
	 * @param dataSourceQualifiers 数据源标识
	 * @return 关闭成功与否的提示信息
	 */
	public String stop(HashSet<DataSourceQualifier> dataSourceQualifiers) {
		// 不指定数据源标识时，关闭所有监听器
		if (CollectionUtils.isEmpty(dataSourceQualifiers)) {
			dataSourceQualifiers = allDataSourceTypes();
		}
		String message = "";
		try {
			for (DataSourceQualifier qualifier : dataSourceQualifiers) {
				message += "\n" + stopListener(qualifier);
			}
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
		}
		return message;
	}

	/**
	 * 关闭监听器
	 * 
	 * @param dataSourceQualifier 数据源标识
	 * @return
	 */
	private String stopListener(DataSourceQualifier dataSourceQualifier) {
		String message = "";
		if (!isRunning(dataSourceQualifier)) {
			message = "索引实时更新服务未开启或已关闭：" + dataSourceQualifier.getQualifier();
			logger.warn(message);
		} else {
			try {
				closeConsumer(dataSourceQualifier);
				message = "索引实时更新服务关闭成功：" + dataSourceQualifier.getQualifier();
				logger.info(message);
			} catch (Throwable e) {
				message = "索引实时更新服务关闭失败：" + dataSourceQualifier.getQualifier();
				logger.error(message, e);
			}
		}
		return message;
	}

	/**
	 * 关闭消息队列的消费者
	 * 
	 * @param dataSourceQualifier 数据源标识
	 */
	private void closeConsumer(DataSourceQualifier dataSourceQualifier) {
        if (!isRunning(dataSourceQualifier)) {
            throw new IllegalStateException("索引实时更新服务未开启或已关闭");
        } else {
            TaskInformation consumer = consumers.get(dataSourceQualifier);
            taskService.remove(consumer.getCode());
            if (!taskService.isStopped()) {
                taskService.stop();
            }
            taskService.start();
            consumers.remove(dataSourceQualifier);
        }
	}

	/**
	 * @param dataSourceQualifier 数据源标识
	 * @return 索引实时更新服务是否正在运行
	 */
	public boolean isRunning(DataSourceQualifier dataSourceQualifier) {
		if (dataSourceQualifier == null) {
			throw new IllegalArgumentException("dataSourceQualifier 为空");
		}
        TaskInformation consumer = consumers.get(dataSourceQualifier);
		return consumer != null && taskService.exist(consumer.getCode());
	}

	/**
	 * @return 所有数据源标识
	 */
	private HashSet<DataSourceQualifier> allDataSourceTypes() {
		HashSet<DataSourceQualifier> dataSourceQualifiers = new HashSet<>();
		DataSourceQualifier[] types = DataSourceQualifier.values();
        dataSourceQualifiers.addAll(Arrays.asList(types));
		return dataSourceQualifiers;
	}

	/**
	 * 对得到的队列消息进行解析，之后根据解析出来的对象，对lucene索引进行添加、更新或删除操作
	 * 
	 * @param luceneMessage 消息
	 */
    private void process(LuceneMessageInterface luceneMessage) {
        logger.info(luceneMessage.toString());
        try {
            ParsedQueueMessage parsedQueueMessage = queueMessageParser.parse(luceneMessage.getTableName(), luceneMessage.getDataId(), luceneMessage.getMethodType(), luceneMessage.getData());
            List<Object> maintainedObjects = indexService.maintainIndexes(parsedQueueMessage);
            logger.info("Maintained... " + maintainedObjects);
        } finally {
            Long id = luceneMessage.getId();
            luceneMessage.setRetryCount(luceneMessage.getRetryCount() + 1);
            if (luceneMessage instanceof com.uas.search.console.b2b.jms.platformb2b.LuceneMessage) {
                com.uas.search.console.b2b.jms.platformb2b.LuceneMessageDao luceneMessageDao = ContextUtils.getBean(com.uas.search.console.b2b.jms.platformb2b.LuceneMessageDao.class);
                luceneMessageDao.save((com.uas.search.console.b2b.jms.platformb2b.LuceneMessage) luceneMessage);
				logger.info("dequeueLuceneMessage.id..." + luceneMessage.getId());
                luceneMessageDao.dequeueLuceneMessage(id);
            } else {
                com.uas.search.console.b2b.jms.platformmanager.LuceneMessageDao luceneMessageDao = ContextUtils.getBean(com.uas.search.console.b2b.jms.platformmanager.LuceneMessageDao.class);
                luceneMessageDao.save((com.uas.search.console.b2b.jms.platformmanager.LuceneMessage) luceneMessage);
				logger.info("dequeueLuceneMessage.id..." + luceneMessage.getId());
                luceneMessageDao.dequeueLuceneMessage(id);
            }
        }
    }
}
